; Tetris89 1.40 macro file by Cyril CLEAUD <cyril.cleaud@gmail.com>
; Last change on January 20, 2005

; --------------- definitions zone ---------------

LinkDelay       equ  250

; link codes (codes are 8bits, in format 0000xxxx)
; the 4 lower bytes are used for value
LinkAddLines    equ  $F0
LinkUpdateBarH  equ  $E0
LinkUpdateBar   equ  $D0
LinkWin         equ  $C0
LinkAbort       equ  $B0
LinkOldConn     equ  $AA
LinkNewConn     equ  $AB
LinkReset       equ  $10



; key ID for the GetKeyStat subprogramm
; this table is aimed to create compatibility between 89 and 92+
; value are now byte sized (space optimization) and must be different
KeyUp           equ  $81
KeyDown         equ  $82
KeyLeft         equ  $83
KeyRight        equ  $84
KeyESC          equ  $85
Key2nd          equ  $86
KeyShift        equ  $87
KeyDiamond      equ  $88
KeyAlpha        equ  $89
KeyHome         equ  $8A
KeyENTER        equ  $8B
KeyF1           equ  $8C
KeyF2           equ  $8D
KeyF3           equ  $8E
KeyF4           equ  $8F

;
; The ti89's keyboard matrix :
; Row     +-------+-------+-------+-------+-------+-------+-------+-------+
;  V  Col>| Bit 7 | Bit 6 | Bit 5 | Bit 4 | Bit 3 | Bit 2 | Bit 1 | Bit 0 |
; +-------+-------+-------+-------+-------+-------+-------+-------+-------+
; | Bit 0 | alpha |diamond| shift |  2nd  | right | down  | left  |  up   |
; | Bit 1 |  F5   | clear |   ^   |   /   |   *   |   -   |   +   | ENTER |
; | Bit 2 |  F4   |backspc|   T   |   ,   |   9   |   6   |   3   |  (-)  |
; | Bit 3 |  F3   |catalog|   Z   |   )   |   8   |   5   |   2   |   .   |
; | Bit 4 |  F2   | mode  |   Y   |   (   |   7   |   4   |   1   |   0   |
; | Bit 5 |  F1   | home  |   X   |   =   |   |   |  EE   |  STO  | APPS  |
; | Bit 6 |       |       |       |       |       |       |       |  ESC  |
; +-------+-------+-------+-------+-------+-------+-------+-------+-------+
;
; The ti92's keyboard matrix :
; Row     +-------+-------+-------+-------+-------+-------+-------+-------+
;  V  Col>| Bit 7 | Bit 6 | Bit 5 | Bit 4 | Bit 3 | Bit 2 | Bit 1 | Bit 0 |
; +-------+-------+-------+-------+-------+-------+-------+-------+-------+
; | Bit 0 |  down | right |   up  |  left |  hand | shift |diamond|  2nd  |
; | Bit 1 |   3   |   2   |   1   |   F8  |   W   |   S   |   Z   | unused|
; | Bit 2 |   6   |   5   |   4   |   F3  |   E   |   D   |   X   | unused|
; | Bit 3 |   9   |   8   |   7   |   F7  |   R   |   F   |   C   |  STO  |
; | Bit 4 |   ,   |   )   |   (   |   F2  |   T   |   G   |   V   | space |
; | Bit 5 |  TAN  |  COS  |  SIN  |   F6  |   Y   |   H   |   B   |   /   |
; | Bit 6 |   P   | ENTER2|   LN  |   F1  |   U   |   J   |   N   |   ^   |
; | Bit 7 |   *   |  APPS | CLEAR |   F5  |   I   |   K   |   M   |   =   |
; | Bit 8 | unused|  ESC  |  MODE |   +   |   O   |   L   | theta |backspc|
; | Bit 9 |  (-)  |   .   |   0   |   F4  |   Q   |   A   | ENTER1|   -   |
; +-------+-------+-------+-------+-------+-------+-------+-------+-------+
;
; See lowlevel.doc in fargoII package for the usage of these matrix.
; Th.FERNIQUE <FFernique@eisti.fr> 17/10/98.

; --------------- asm macros code zone ---------------

; Copy a part of the LCD screen from one plane to another (byte per byte : quick)
; a0 : source plane
; a1 : destination plane
; d0 : x1 (1 equ 8bits)
; d1 : y1
; d2 : x2 (1 equ 8bits)
; d3 : y2
; Released by cyril.cleaud@gmail.com - January 20, 2005
LCD_Copy8:
 movem.l a0-a1/d0-d7,-(a7)
 clr.w   d5                            ; Height loop
_lcd_copy8_loopheight:
  clr.w   d6                           ; Width loop
 _lcd_copy8_loopwidth: 

    cmp.w   d1,d5                      ; if(d5>=d1) 
    blt     _lcd_copy8_notok 
     cmp.w   d3,d5                     ; if(d5<=d3)
     bgt     _lcd_copy8_notok 
      cmp.w   d0,d6                    ; if(d4>=d0)
      blt     _lcd_copy8_notok 
       cmp.w   d2,d6                   ; if(d4<=d2)
       bgt     _lcd_copy8_notok 
       
       move.b  (a0),(a1)
      
   _lcd_copy8_notok:

  
  addq    #1,a0                        ; Increment LCD source pointer
  addq    #1,a1                        ; Increment LCD destination pointer
  addq    #1,d6                        ; Width loop
  cmp.w   #30,d6                       ; 30 bytes per line (only 20 are used on 89)
  bne     _lcd_copy8_loopwidth
 
 addq    #1,d5                         ; Height loop
 cmp.w   #LCD_HEIGHT,d5
 bne     _lcd_copy8_loopheight
 
 movem.l (a7)+,a0-a1/d0-d7
 rts

; --------------- macros zone ---------------

; GetKey value_write,value_read
GetKey MACRO
 move.w  \1,($600018)
 movem.l d0,-(a7)       ; save d0
 moveq   #9,d0			; just 10 nops to let hardware react ...
getkeywait
 nop
 dbra     d0,getkeywait
 movem.l (a7)+,d0       ; restore d0
 move.b  ($60001B),\2
 ENDM

LCD_Copy_Zone8 MACRO    ; plane_from,plane_to,x1,y1,x2,y2
 movem.l a0-a1/d0-d7,-(a7)
 move.l  \1,a0
 move.l  \2,a1
 move.l  \3,d0
 move.l  \4,d1
 move.l  \5,d2
 move.l  \6,d3
 bsr     LCD_Copy8
 movem.l (a7)+,a0-a1/d0-d7
 ENDM

;; WriteStr x,y,style,LPSTR
;WriteStr MACRO
; move.w  \3,-(a7)
; pea     \4
; move.w  \2,-(a7)
; move.w  \1,-(a7)
; jsr     tios::DrawStrXY
; lea     10(a7),a7
; ENDM
;
;; WriteStr x,y,style,LPSTR_with_address_reg
;WriteStrA MACRO
; move.w  \3,-(a7)
; move.l  \4,-(a7)
; move.w  \2,-(a7)
; move.w  \1,-(a7)
; jsr     tios::DrawStrXY
; lea     10(a7),a7
; ENDM
;
;; SetFont number_of_font
;SetFont MACRO
; move.w  \1,-(a7)
; jsr     tios::FontSetSys
; lea     2(a7),a7
; ENDM