;                     Tetris 1.21 89/92+
;
;             originally by Jimmy Mrdell for TI 92
;            ported from the 92 to 89 by Joel Jordan
;    upgraded for 89/92+ by Cyril Cleaud <cyril.cleaud@gmail.com>
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Version 0.70  - added two player support
;                 graphics data fixed (no clipping at bottom of sprites)
; Version 0.71  - recompiled for new version of PlusShell 1.0a and DoorsOS 1.0
; Version 0.72  - now works with new roms 2.0x
;                 the teacher key (home) doesn't crash the calc anymore
;                 fixed the GameOver dialog
; Version 0.75  - fixed highscores for ROMS 2.0x
;                 grayscales work on HW2
; Version 0.76a - added B mode (1P only)
;                 added statistics about player lines
; Version 0.76b - minor bugfixes in number conversion
; Version 0.76c - minor bugfixe again
;                 changed main logo to fit 89 screen width
; Version 0.77  - added compatibility to 92+
;                 added statistics about random piece
; Version 0.77a - changed random piece generator
; Version 0.77b - increased speed to move piece horizontaly (level 16 is now easyly playable)
; Version 0.77c - no need of gray4lib now
;                 link part is all rewritten, compatible and autodetect old or cheated versions
;                 score part is all rewritten, now store lines and level
; Version 0.77d - no need of util now (all its exportations are in this file)
;                 corrected the game aborted bug in 2P mode
;                 corrected the font bug with show_dialog (changed library reference)
; Version 1.40  - implemented same speed on HW1 and HW2
;                 minor GFX bugfixes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Gray scales work on HW2 with latest libraries included in TeOS package or PreOS package
; File last modified on January 19th, 2005 by Cyril Cleaud <cyril.cleaud@gmail.com>
;

    include "..\tios.h"             ; TiOS definitions with some macro
    include "..\graphlib.h"
    include "tetmacr.h"             ; Tetris MACRO definitions

    xdef _main
    xdef _comment
    xdef _ti89ti
    xdef _v200
    xdef _ti89
    xdef _ti92plus


; Constant timer value
timerHW1       equ 20
ltimerHW1      equ 50
rtimerHW1      equ 50

timerHW2       equ 20
ltimerHW2      equ 30
rtimerHW2      equ 30

HW2timer_mul   equ 34           ; timerHW2 = timer * timer_mul/timer_div
HW2timer_div   equ 50


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;    Code zone     ;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


_main:
 move.b  HW_VERSION,d0
 cmp.b   #1,d0
 beq     Init                   ; calc is HW1 then skip timer adjustment for HW2+
 
 move.w  #timerHW2,timer_value  ; timer adjustment for movement
 move.w  #ltimerHW2,ltimer_value
 move.w  #rtimerHW2,rtimer_value
 
 lea	 levels(PC),a0          ; timer adjustment for level speed
 moveq   #30,d1                 ; 30 speed level to adjust
HW2AdjustLvlSpeed:
 clr.l   d0
 move.w  (a0),d0
 mulu	 #HW2timer_mul,d0
 divu    #HW2timer_div,d0
 and.w   #$FFFF,d0
 move.w  d0,(a0)+
 dbra    d1,HW2AdjustLvlSpeed


; install timer subprogramm
Init:
 lea board,a5
 sf	 intflag
 move.l  ($64),oldint1
 move.l  #Int1Handler,($040064)
 jsr graphlib::gray4

; find calculator type (89/92+)
 clr.b   is92p
 tst.b   CALCULATOR
 beq MainRepeat                 ; calc is a 89 then continue programm
 lea     ti92pdialog(PC),a6     ; calc is a 92+ then warn the user
 move.b  #1,is92p               ; update flag 92+
 bsr ShowDialog
 SetFont #0
 WriteStr #22,#54,#4,badcalctxt
 bsr CopyDialogBox
 move.w  #1000,timer            ; wait for 1000 ticks of int1
BadCalcWait:
 tst.w   timer
 bne BadCalcWait

MainRepeat:
 bsr Titel
 bsr idle_loop
 cmp.w	 #264,d0
 beq Done
MainRepeatMenu:
 bsr Menu
 cmp.w	 #264,d0
 beq Done
 bsr NewGame

; main game loop ...
Main:                           ; 2P flag = 0 ?
 tst.w	 twopl                  ; yes : 1P mode => jump GetKeyInfo
 beq GetKeyInfo                 ; we're now in 2P mode
 bsr ReceiveByte                ; d0 = byte received
 move.w  d0,d1                  ; d1 = d0 = byte received
 tst.w   d0
 beq GetKeyInfo                 ; no byte received => continue game
 and.w	 #$0F,d1                ; clear 28 high bits of d1
 and.w   #$F0,d0                ; keep only link code
 cmp.w	 #LinkAddLines,d0
 beq PenaltyRows
 cmp.w	 #LinkUpdateBarH,d0
 beq BarInfoH
 cmp.w	 #LinkUpdateBar,d0
 beq BarInfo
 cmp.w   #LinkAbort,d0
 beq GameAbort
 cmp.w	 #LinkWin,d0
 bne GetKeyInfo
 lea	 windialog(PC),a6
 bra EndGame

; this part is for 2P only
BarInfoH:
 moveq	 #16,d1
BarInfo:
 bsr ShowBar
 bra GetKeyInfo
PenaltyRows:
 bsr RemoveCurPiece
Put_PR:
 bsr TrashRow
 tst.w	 y
 beq NextPR
 subq.w  #1,y
NextPR:
 dbra	d1,Put_PR
 bsr ShowCurPiece
 bsr SendBarInfo


GetKeyInfo:
 bsr GetKeyStat
 bsr KeyTrans
 move.w  x(PC),d4
 move.w  y(PC),d5
 move.w  rot(PC),d6
 bsr CheckDown
 bsr CheckLeftRight
 bsr CheckRotate
 bsr CheckDrop
 clr.l	 ltimer
 sf	 upflag
 sf	 downflag

; test if in B mode the total number of line is done ...
 tst.w	 bmode                  ; bmode = 0 ?
 beq endtestlinebmode           ; yes : not in B mode => add lines

 movem.l d0-d1/a0-a1,-(a7)
 move.w  maxlines,d0
 sub.w   lines,d0
 tst.w   d0
 movem.l (a7)+,d0-d1/a0-a1
 bgt endtestlinebmode
 move.w	 maxlines,lines
 lea	 windialog(PC),a6       ; player win (B mode)
 bra EndGame
endtestlinebmode:


 include "tetdbg1.asm"


 bsr ShowScore			        ; update score and lines statistics
 cmp.b   #KeyESC,KeyEqu         ; (ESC)
 beq GameAbort
 tst.w	 twopl
 bne EndKeyCheck
 cmp.b   #KeyAlpha,KeyEqu       ; (alpha)
 beq Pause
 cmp.b   #KeyHome,KeyEqu        ; (home)
 beq TeacherKey
EndKeyCheck:



CheckTimer:
 tst.w	 timer
 bne Main
ShortWait:
 move.w  timer_value,d0
 move.w  d0,timer
Wait:
 tst.w	 timer
 bne Wait
 bra MoveDown


RetEKC:
 addq	 #4,a7
 bra EndKeyCheck

CheckRotate:
 cmp.b   #KeyUp,KeyEqu          ; (up)
 beq TryRot
SkipUpCheck:
 cmp.b   #Key2nd,KeyEqu         ; (2nd)
 beq TryRot
 cmp.b   #KeyShift,KeyEqu       ; (shift)
 bne ClearUp
 addq	 #2,d6
TryRot:
 clr.l	 ltimer
 sf	 downflag
 addq	 #1,d6
 and.w	 #3,d6
 tst.b	 upflag
 bne RetEKC
 st	 upflag
 lea	 tmpp(PC),a0
 move.w  curPN(PC),d0
 move.w  d6,d1
 bsr Uncrunch
 bsr TestMoveA0
 bne RetEKC
 bsr RemoveCurPiece
 move.w  d6,rot
 bsr UncrunchCurPiece
 bsr ShowCurPiece
 bra RetEKC
ClearUp:
 sf	 upflag
 rts

CheckLeftRight:
 cmp.b   #KeyLeft,KeyEqu        ; (left)
 bne CheckRight
 clr.w	 rtimer
 sf	 downflag
 tst.w	 ltimer
 bne RetEKC
 move.w  ltimer_value,d0        ;  left timer (was 60)
 move.w  d0,ltimer
 subq	 #1,d4
 bsr TestMove
 bne RetEKC
 bsr RemoveCurPiece
 move.w  d4,x
 bsr ShowCurPiece
 bra RetEKC
CheckRight:
 clr.w	 ltimer
 cmp.b   #KeyRight,KeyEqu       ; (right)
 bne ClearRight
 sf	 downflag
 tst.w	 rtimer
 bne RetEKC
 move.w  rtimer_value,d0        ; right timer (was 60)
 move.w  d0,rtimer
 addq	 #1,d4
 bsr TestMove
 bne RetEKC
 bsr RemoveCurPiece
 move.w  d4,x
 bsr ShowCurPiece
 bra RetEKC
ClearRight:
 clr.w	 rtimer
 rts

CheckDown:
 tst.w	 disDflag
 bne Return
 cmp.b   #KeyDown,KeyEqu        ; (down)
 bne Return
 tst.b	 downflag
 bne RetEKC
 addq.l  #1,score
 addq	 #4,a7
 bsr ShowScore                  ; update score
 bra ShortWait

CheckDrop:
 cmp.b   #KeyDiamond,KeyEqu     ; (diamond)
 bne ClearDown
TryDrop:
 tst.b	 downflag
 bne RetEKC
Drop:
 addq	 #1,d5
 addq.l  #1,score
 bsr TestMove
 beq Drop
 bsr RemoveCurPiece
 move.w  d5,y
 subq.w  #1,y
 bsr ShowCurPiece
 addq	 #4,a7
 bra StorePiece
ClearDown:
 sf	 downflag
 rts

MoveDown:
 move.w  levtimer(PC),timer
 move.w  x(PC),d4
 move.w  y(PC),d5
 addq	 #1,d5
 bsr TestMove
 bne StorePiece
 bsr RemoveCurPiece
 move.w  d5,y
 bsr ShowCurPiece
 bra Main
StorePiece:
 subq	 #1,d5
 moveq	 #3,d7
 lea	 curp(PC),a0
StoreSq:
 move.w  (a0)+,d2
 move.w  (a0)+,d3
 add.w	 d5,d2
 lsl.w	 #4,d2
 add.w	 d3,d2
 add.w	 d4,d2
 st	 0(a5,d2)
 dbra	 d7,StoreSq
 move.l  a5,a0
 lea	 rows(PC),a1
 moveq	 #17,d0
 clr.l	 d2
 clr.l	 d5
CheckRow:
 moveq	 #15,d1
CheckRCol:
 tst.b	 0(a0,d1)
 beq CheckNextRow
 dbra	 d1,CheckRCol
 addq	 #1,d5
 move.w  d2,(a1)+
CheckNextRow:
 lea	 16(a0),a0
 addq	 #1,d2
 dbra	 d0,CheckRow
 bsr ShowScore
 tst.w	 d5                     ; d5 = 0 ?
 beq NewP                       ; yes => jump
 add.w	 d5,lines	            ; now d5={1,2,3,4}

; update for statistics about player lines
 lea	 Sline-2(PC),a0
 add     d5,a0		            ; double add because values are 16bits
 add     d5,a0
 addq.w	 #1,(a0)

 clr.l	 d0
 move.w  lines(PC),d0
 divu	 #10,d0
 cmp.w	 level(PC),d0
 blt SameLevel
 move.w  d0,level
 lea	 levels(PC),a0
 lsl.w	 #1,d0
 move.w  0(a0,d0),levtimer
SameLevel:
 subq	 #1,d5
 lea	 scoring(PC),a0
 move.w  d5,d0
 lsl.w	 #1,d0
 move.w  0(a0,d0),d0
 move.w  level(PC),d1
 addq	 #1,d1
 mulu	 d1,d0
 add.l	 d0,score
 moveq	 #3,d6
Flash:
 move.w  d5,d0
 lea	 rows(PC),a0
 move.l graphlib::plane0,a1
 move.l  graphlib::plane1,a2
FlashRows:
 move.w  (a0)+,d1
 subq	 #2,d1
; Changed this from 240 to 180 because the rows are only 6 pixels high*30 bytes across = 180 bytes
 mulu	 #180,d1
 addq.w	 #1,d1
 moveq	 #5,d2
FlashLine:
 move.w  d1,d3
; I changed this next line from 4 to 9 because my method puts the first
; usuable position on an odd address, and i get address faults if I do this
; with word or long operations like the 92 uses for some reason
 move.w	 #9,d4
FlashWord:
; substituted 10 byte or's for 5 word or's
 eori.b #$FF,0(a1,d3.w)
 eori.b #$FF,0(a2,d3.w)
 addq.w #1,d3
 dbra	 d4,FlashWord
 add.w	 #30,d1
 dbra	 d2,FlashLine
 dbra	 d0,FlashRows
 move.w  #40,timer
FlashPause:
 tst.w	 timer
 bne FlashPause
 dbra	 d6,Flash
 move.w  d5,d0
 lea	 rows(PC),a0
MoveLines:
 move.l graphlib::plane0,a1
 move.l  graphlib::plane1,a2
 move.w  (a0)+,d1
 move.w  d1,d4
 subq	 #2,d1
 move.w  d1,d3

; Multiply by 6*30 = 180 bytes per block row...changed from 240
 mulu	 #180,d1
; add 150 instead of 220...this is because the rows begin at a different
; x coordinate than on the 92
 add.w	 #150,d1
 move.w  d1,d2
; subtract 180 instead of 240--6*30=180...
 sub.w	 #180,d2
 mulu    #6,d3
 subq	 #1,d3
MoveGFXRows:
; I had to expand these cuz the 68K won't let me do long operations
; starting at an odd address
 move.b  10(a1,d2),10(a1,d1)
 move.b  10(a2,d2),10(a2,d1)
 move.b  1(a1,d2),1(a1,d1)
 move.b  1(a2,d2),1(a2,d1)
 move.b  2(a1,d2),2(a1,d1)
 move.b  2(a2,d2),2(a2,d1)
 move.b  3(a1,d2),3(a1,d1)
 move.b  3(a2,d2),3(a2,d1)
 move.b  4(a1,d2),4(a1,d1)
 move.b  4(a2,d2),4(a2,d1)
 move.b  5(a1,d2),5(a1,d1)
 move.b  5(a2,d2),5(a2,d1)
 move.b  6(a1,d2),6(a1,d1)
 move.b  6(a2,d2),6(a2,d1)
 move.b  7(a1,d2),7(a1,d1)
 move.b  7(a2,d2),7(a2,d1)
 move.b  8(a1,d2),8(a1,d1)
 move.b  8(a2,d2),8(a2,d1)
 move.b  9(a1,d2),9(a1,d1)
 move.b  9(a2,d2),9(a2,d1)
 sub.w	 #30,d1
 sub.w	 #30,d2
 dbra	 d3,MoveGFXRows

; changed a 7 to 5 (8 rows per block vs 6 rows per block)
 moveq	 #5,d3
ClearTopRows:
 clr.b	 10(a1,d1)
 clr.b	 10(a2,d1)
 clr.b	 1(a1,d1)
 clr.b	 1(a2,d1)
 clr.b	 2(a1,d1)
 clr.b	 2(a2,d1)
 clr.b	 3(a1,d1)
 clr.b	 3(a2,d1)
 clr.b	 4(a1,d1)
 clr.b	 4(a2,d1)
 clr.b	 5(a1,d1)
 clr.b	 5(a2,d1)
 clr.b	 6(a1,d1)
 clr.b	 6(a2,d1)
 clr.b	 7(a1,d1)
 clr.b	 7(a2,d1)
 clr.b	 8(a1,d1)
 clr.b	 8(a2,d1)
 clr.b	 9(a1,d1)
 clr.b	 9(a2,d1)
 sub.w	 #30,d1
 dbra	 d3,ClearTopRows
 move.l  a5,a1
 lsl.w	 #4,d4
 add.w	 d4,a1
 move.l  a1,a2
 lea	 16(a2),a2
 lsr.w	 #2,d4
 subq	 #1,d4
UpdateBoard:
 move.l  -(a1),-(a2)
 dbra	 d4,UpdateBoard
 dbra	 d0,MoveLines
 tst.w	 twopl
 beq NewP
 tst.w	 d5
 beq NewP
 move.w  llflag(PC),d0
 lsr.w	 #3,d0
 sub.w	 d0,d5
 move.w  d5,d0
 or.w	 #LinkAddLines,d0
 bsr SendByte                   ; send new lines to other calc
NewP:
 tst.w	 twopl
 beq NoBarInfo
 bsr SendBarInfo
NoBarInfo:
 bsr ShowScore
 bsr NewPiece
 moveq	 #6,d4
 clr.l	 d5
 bsr TestMove
 beq Main
GameOver:
 tst.w	 twopl
 beq ShowGameOver
 move.b  #LinkWin,d0
 bsr SendByte                   ; send win message to other calc
ShowGameOver:
 lea	 losedialog(PC),a6
EndGame:
 bsr ShowScore
 bsr ShowDialog
 SetFont #0
 WriteStr #30,#55,#4,entertxt
 bsr CopyDialogBox
WaitQRel:
 bsr GetKeyStat
 bsr KeyTrans
 cmp.b   #KeyENTER,KeyEqu       ; (ENTER)
 bne WaitQRel
 sf	 intflag
CheckGameOver:
 jsr graphlib::clear_dialog     ; clear dialog
 bsr CopyDialogBox
 bsr ShowInfoText               ; restore displayed statistics
 bsr ShowScore

; now display game statistics
 movem.l d0-d4/a0-a2,-(a7)
 moveq	 #100,d1                ; clear screen game zone and bar info
 move.l  graphlib::plane0,a1    ; clear in plane0
 move.l  graphlib::plane1,a2    ; clear in plane1
P_MoveRowBis:
 moveq	 #12,d2
P_MoveWordBis:
 clr.b	 (a1)+
 clr.b	 (a2)+
 dbra	 d2,P_MoveWordBis
 lea	 17(a1),a1
 lea	 17(a2),a2
 dbra	 d1,P_MoveRowBis
 bsr PauseShowStat
 movem.l (a7)+,d0-d4/a0-a2

 tst.w	 twopl                  ; 2 players ? => do not ask for name
 bne MainRepeat
 tst.w	 bmode                  ; B mode ? => do not ask for name
 bne MainRepeat

; try to insert new score
 move.l  score(PC),d0
 cmp.l	 hiscore+72,d0
 ble MainRepeat                 ; the score is too small and can't be inserted

; we must insert the new score
 lea	 hiscore+16(PC),a0
 moveq	 #2,d1
CheckPl:
 cmp.l	 (a0),d0
 bgt PlFound
 lea	 28(a0),a0
 dbra	 d1,CheckPl
PlFound:
 lea	 hiscore+84(PC),a0
 lea	 hiscore+56(PC),a3
 tst.w	 d1
 beq NoHiMove
 mulu	 #7,d1
 subq	 #1,d1
MoveHiTable:
 move.l  -(a3),-(a0)
 dbra	 d1,MoveHiTable
NoHiMove:
 move.l  d0,16(a3)              ; save score in highscore
 move.l  d0,-(a7)
 move.w  lines,d0
 and.l   #$FFFF,d0
 move.l  d0,20(a3)              ; save lines in highscore
 move.w  level,d0
 move.l  d0,24(a3)              ; save level in highscore
 move.l  (a7)+,d0

 jsr graphlib::gray2            ; disable gray 4
 bsr ClrScr
 SetFont #2
 WriteStr #16,#12,#4,ent1txt

 SetFont #1
 movem.l d0-d2/a0-a1,-(a7)
 lea	 ent4txt+19(PC),a0
 move.l  score(PC),d0
 moveq	 #6,d1
 bsr ConvStr
 sub.l   #13,a0
 WriteStrA #17,#47,#4,a0
 movem.l (a7)+,d0-d2/a0-a1
 WriteStr #2,#37,#4,ent2txt
 WriteStr #30,#64,#4,ent3txt
 move.w  #30,d3
 moveq	 #75,d1
 clr.l	 d4
 clr.w   (tios::kb_vars+$1C)

Input:
 move    d3,d0
 move.w  #95,d2
 bsr WriteChar
 bsr idle_loop
 cmp.w	 #13,d0			        ; (enter)
 beq InputDone
 cmp.w	 #264,d0		        ; (ESC)
 beq InputDoneESC
 cmp.w	 #257,d0		        ; (backspace)
 beq BackSpace
 cmp.w   #263,d0                ; (clear)
 beq InputClear
 cmp.w	 #32,d0                 ; (espace)
 bcs Input
 cmp.w	 #256,d0                ; ??
 bcc Input
 cmp.w	 #15,d4                 ; ??
 beq Input
 move.b  d0,(a3)+
 move.w  d0,d2
 move.w  d3,d0
 bsr WriteChar
 addq	 #6,d3
 addq	 #1,d4
 bra Input
BackSpace:
 tst.w	 d4
 beq Input
 clr.b	 -(a3)
 move.w  d3,d0
 moveq	 #32,d2
 bsr WriteChar
 subq	 #6,d3
 move.w  d3,d0
 bsr WriteChar
 subq	 #1,d4
 bra Input
InputClear:
 move.w  #15,d5
ClearName:
 tst.w	 d4
 beq Input
 clr.b	 -(a3)
 move.w  d3,d0
 moveq	 #32,d2
 bsr WriteChar
 subq	 #6,d3
 move.w  d3,d0
 moveq	 #32,d2
 bsr WriteChar
 subq	 #1,d4
 bra ClearName
InputDoneESC:
 sub     d4,a3                  ; adjust a3 to start of memory zone
 move.l  #$4E6F206E,(a3)		; hex code for 'No n'
 addq	 #4,a3
 move.l  #$616D6500,(a3)		; hex code for 'ame\0'
 addq	 #3,a3
 addq	 #7,d4
InputDone:
 tst.w   d4
 beq InputDoneESC
 clr.b	 (a3)
 jsr graphlib::gray4            ; enable gray 4
 bra MainRepeat

; we jump here if a player aborts the game (ESC)
GameAbort:
 tst.w	 twopl
 beq ShowGameAbort
 move.b  #LinkAbort,d0
 bsr SendByte                   ; send abort message to other calc
 move.l  #999,d0                ; just 1000 nops to let hardware react ...
get_send_wait:
 nop
 dbra     d0,get_send_wait
 bsr ReceiveByte				; receive any byte from other calc
ShowGameAbort:
 lea	 abortdialog(PC),a6
 bra EndGame

ShowScore:
 movem.l d0-d2/a0-a1,-(a7)
 SetFont #0		
; display single line number done
 lea	 tmpstr+3(PC),a0
 clr.l	 d0
 move.w  Sline(PC),d0
 moveq	 #3,d1
 bsr ConvStr
 WriteStrA #140,#2,#4,a0
; display double line number done
 lea	 tmpstr+3(PC),a0
 clr.l	 d0
 move.w  Dline(PC),d0
 moveq	 #3,d1
 bsr ConvStr
 WriteStrA #140,#8,#4,a0
; display triple line number done
 lea	 tmpstr+3(PC),a0
 clr.l	 d0
 move.w  Tline(PC),d0
 moveq	 #3,d1
 bsr ConvStr
 WriteStrA #140,#14,#4,a0
; display tetris line number done
 lea	 tmpstr+3(PC),a0
 clr.l	 d0
 move.w  TTline(PC),d0
 moveq	 #3,d1
 bsr ConvStr
 WriteStrA #140,#20,#4,a0
 
 SetFont #1
; display current level	number
 lea	 tmpstr+2(PC),a0
 clr.l	 d0
 move.w  level(PC),d0
 moveq	 #2,d1
 bsr ConvStr
 WriteStrA #140,#30,#4,a0
; display current score
 lea	 tmpstr+6(PC),a0
 move.l  score(PC),d0
 moveq	 #6,d1
 bsr ConvStr
 WriteStrA #116,#68,#4,a0
; display lines	number (done / to do [if in B mode])
 lea	 tmpstr+6(PC),a0
 clr.l	 d0
 move.w  lines(PC),d0

; warning : new compute for B mode
 tst.w	 bmode              ; bmode = 0 ?
 beq ShowScendadjustNBline  ; Yes : not in B mode
 clr.l   d1
 move.w  d0,d1
 move.w  maxlines,d0
 sub.w   d1,d0
ShowScendadjustNBline:
 moveq	 #6,d1
 bsr ConvStr
 WriteStrA #116,#48,#4,a0

; changed the addresses here a bit to make up for the changes in position
 move.l graphlib::plane0,a0
 lea     $C(a0),a0
 move.l  graphlib::plane1,a1
 add.w	 #$c,a1
 moveq	 #80,d0
SS_CopyRow:
 moveq	 #8,d1
SS_CopyCol:
 move.b  (a0)+,(a1)+
 dbra	 d1,SS_CopyCol
 lea	 21(a0),a0
 lea	 21(a1),a1
 dbra	 d0,SS_CopyRow
 movem.l (a7)+,d0-d2/a0-a1
 rts

; convert a number in range 0 - 655359 only
; it runs without bug now ! (if d0>655359 then d0=655359)
; a0 : @mem of string
; d0 : the number to convert in range 0-655359
; d1 : number of char in converted string
ConvStr:
 cmp.l	 #$9FFFF,d0
 ble _ConvStrStart
 move.l	 #$9FFFF,d0
_ConvStrStart:
 movem.l d0-d2,-(a7)
 clr.b	 (a0)
_RepConv:
 divu	 #10,d0
 move.l  d0,d2
 swap	 d2
 add.b	 #'0',d2
 move.b  d2,-(a0)
 subq	 #1,d1
 and.l	 #$FFFF,d0
 bne _RepConv
 tst.w	 d1
 beq _CS_Done
 subq	 #1,d1
_FillOut:
 move.b  #32,-(a0)
 dbra	 d1,_FillOut
_CS_Done:
 movem.l (a7)+,d0-d2
 rts


; this has been converted 5/15/98
TrashRow:
 movem.l d0-d4/a0-a4,-(a7)
 move.l graphlib::plane0,a2
 move.l  graphlib::plane1,a3
 moveq	 #1,d1
 move.w  #181,d2
 moveq	 #89,d3
MoveUpGFXRows:
; stupid byte align strikes again!
 move.b  0(a2,d2),0(a2,d1)
 move.b  0(a3,d2),0(a3,d1)
 move.b  1(a2,d2),1(a2,d1)
 move.b  1(a3,d2),1(a3,d1)
 move.b  2(a3,d2),2(a3,d1)
 move.b  2(a2,d2),2(a2,d1)
 move.b  3(a3,d2),3(a3,d1)
 move.b  3(a2,d2),3(a2,d1)
 move.b  4(a3,d2),4(a3,d1)
 move.b  4(a2,d2),4(a2,d1)
 move.b  5(a3,d2),5(a3,d1)
 move.b  5(a2,d2),5(a2,d1)
 move.b  6(a3,d2),6(a3,d1)
 move.b  6(a2,d2),6(a2,d1)
 move.b  7(a3,d2),7(a3,d1)
 move.b  7(a2,d2),7(a2,d1)
 move.b  8(a3,d2),8(a3,d1)
 move.b  8(a2,d2),8(a2,d1)
 move.b  9(a3,d2),9(a3,d1)
 move.b  9(a2,d2),9(a2,d1)
 add.w	 #30,d1
 add.w	 #30,d2
 dbra	 d3,MoveUpGFXRows
 add.w	 d1,a2
 add.w	 d1,a3
 move.l  a5,a0
 move.l  a0,a4
 add.w	 #16,a0
 moveq	 #67,d1
MoveUpRows:
 move.l  (a0)+,(a4)+
 dbra	 d1,MoveUpRows
 addq	 #3,a4
 moveq	 #9,d3
TR_Fill:
 st	 0(a4,d3)
 dbra	 d3,TR_Fill
 moveq	 #4,d3
MakeHoles:
 moveq	 #10,d0
 bsr random
 clr.b	 0(a4,d0)
 dbra	 d3,MakeHoles
 moveq	 #9,d3
 clr.l	 d2
TR_PutSquares:
 lea	 empty(PC),a0
 tst.b	 0(a4,d2)
 beq TR_PutSquare
 lea	 pgfx(PC),a0
 moveq	 #7,d0
 bsr random
 lsl.w	 #4,d0
 add.w	 d0,a0
TR_PutSquare:
 move.l  a2,a1
 bsr PutSprite6
 move.l  a3,a1
 bsr PutSprite6
 addq	 #1,d2
 addq	 #1,a2
 addq	 #1,a3
 dbra	 d3,TR_PutSquares
 movem.l (a7)+,d0-d4/a0-a4
 rts

TestMove:
 lea	 curp(PC),a0
TestMoveA0:
 movem.l d2-d3/d7,-(a7)
 moveq	 #3,d7
CheckSq:
 move.w  (a0)+,d2
 move.w  (a0)+,d3
 add.w	 d5,d2
 lsl.w	 #4,d2
 add.w	 d3,d2
 add.w	 d4,d2
 tst.b	 0(a5,d2)
 bne TM_Done
 dbra	 d7,CheckSq
 cmp.w	 d0,d0
TM_Done:
 movem.l (a7)+,d2-d3/d7
 rts


; sub programm added to display a piece anywhere
; d0 = piece ID
; d2 = X coordinate
; d3 = Y coordinate
DisplayPieceXYn:
 movem.l d0-d7/a0-a6,-(a7)      ; save all reg
 move.w  d0,d6
 move.w  nextPN,d7              ; save variable
 move.w  d0,nextPN
 clr.l	 d1
 movem.l d0-d1/a0-a1,-(a7)
 lea     nextptmp(PC),a1        ; target
 lea     nextp(PC),a0           ; source
 move.w  #15,d0                 ; # of bytes
 bsr memcopy                    ; save nextp(PC) in nextptmp(PC) to keep piece state
 movem.l (a7)+,d0-d1/a0-a1
 lea	 nextp(PC),a0
 bsr Uncrunch
 move.w  d2,d0                  ; X coordinate
 move.w  d3,d1                  ; Y coordinate
 bsr ShowPiece
 movem.l d0-d1/a0-a1,-(a7)
 lea     nextp(PC),a1           ; target
 lea     nextptmp(PC),a0        ; source
 move.w  #15,d0                 ; # of bytes
 bsr memcopy                    ; save nextptmp(PC) in nextp(PC) to restore piece state
 movem.l (a7)+,d0-d1/a0-a1
 move.w  d7,nextPN              ; restore variable
 movem.l (a7)+,d0-d7/a0-a6      ; restore all reg
 rts


; sub programm added to display a piece statistics
; d0 = piece ID
; d2 = X coordinate of piece
; d3 = Y coordinate of piece
ShowPieceStat:
 movem.l d0-d7/a0-a6,-(a7)
 and.l   #$FFFF,d0              ; piece ID in 16bits
 lea	 PieceID0(PC),a0
 add     d0,a0		            ; double add because values are 16bits, d0={0,1,2,3,4,5,6}
 add     d0,a0
 move.w	 (a0),d0                ; get freq of piece
 moveq	 #3,d1
 lea	 tmpstr+3(PC),a0
 bsr ConvStr
 mulu    #8,d2                  ; compute (X,Y) text coordinates
 add.w   #14,d2
 mulu    #6,d3
 add.w   #8,d3
 movem.l d0-d7,-(a7)
 WriteStrA d2,d3,#4,a0
 movem.l (a7)+,d0-d7
 lsr     #3,d2                  ; d2/8 --> d2
 move.l  d2,d4
 addq    #2,d4                  ; d2+2 --> d4
 move.l  d3,d5
 addq    #6,d5                  ; d3+6 --> d5
 LCD_Copy_Zone8 graphlib::plane0,graphlib::plane1,d2,d3,d4,d5
 movem.l (a7)+,d0-d7/a0-a6
 bsr DisplayPieceXYn
 rts

; copy d0 bytes from a0 to a1
memcopy:
 move.b  (a0)+,(a1)+
 dbra    d0,memcopy
 rts
 
; player pause the game
Pause:
 move.b  #1,ispaused
 bsr PausePreShowStat
 clr.b   ispaused
 bra Main

; player press Teacher Key when in pause
PauseTKey:
 bsr ShutDown
 bra WaitPRel

; we show statistics about piece, paused the game
PauseShowStat:
 tst.b   ispaused               ; pause flag = 0 ?
 beq NotInPause                 ; yes : not do display pause text
 SetFont #2
 WriteStr #0,#0,#4,pausetxt
 bra EndDisplayPauseTxt
NotInPause:
 bsr ClearNextPiece
EndDisplayPauseTxt:
 SetFont #1
 WriteStr #14,#12,#4,statistxt
 LCD_Copy_Zone8 graphlib::plane0,graphlib::plane1,#0,#0,#14,#19

 
 SetFont #0                     ; small size
; statistics : show piece frequence
 move.w  #0,d0
 move.l  #2,d2                  ; X coordinate
 move.l  #4,d3                  ; Y coordinate
 bsr ShowPieceStat
 move.w  #1,d0
 move    #9,d2                  ; X coordinate
 bsr ShowPieceStat
 move.w  #2,d0
 move.w  #2,d2                  ; X coordinate
 add.w   #3,d3                  ; Y coordinate
 bsr ShowPieceStat
 move.w  #3,d0
 move.w  #9,d2                  ; X coordinate
 bsr ShowPieceStat
 move.w  #4,d0
 move.w  #2,d2                  ; X coordinate
 add.w   #3,d3                  ; Y coordinate
 bsr ShowPieceStat
 move.w  #5,d0
 move.w  #9,d2                  ; X coordinate
 bsr ShowPieceStat
 move.w  #6,d0
 move.w  #2,d2                  ; X coordinate
 add.w   #3,d3                  ; Y coordinate
 bsr ShowPieceStat
 WriteStr #50,#78,#4,totaltxt
 clr.l   d0                     ; compute total number of piece given
 move.l  #6,d1
 lea	 PieceID0(PC),a0
TotalPiece:
 add.w   (a0),d0
 addq.l  #2,a0
 dbra    d1,TotalPiece
 lea	 tmpstr+3(PC),a0        ; display total piece given
 moveq	 #3,d1
 bsr ConvStr
 WriteStrA #65,#84,#4,a0
 LCD_Copy_Zone8 graphlib::plane0,graphlib::plane1,#6,#78,#11,#89
 WriteStr #10,#95,#4,entertxt
 LCD_Copy_Zone8 graphlib::plane0,graphlib::plane1,#0,#95,#14,#99

 cmp.b   #2,ispaused            ; pause flag = 2 ?
 beq PauseTKey                  ; yes : shutdown calculator
 st intflag
WaitPRel:
 bsr GetKeyStat
 bsr KeyTrans
 cmp.b   #KeyHome,KeyEqu        ; test for teacher key
 beq PauseTKey
 cmp.b   #KeyENTER,KeyEqu       ; wait until ENTER is pressed
 bne WaitPRel
 sf	 intflag
 tst.b   ispaused               ; pause flag = 0 ?
 bne PauseShowStatRTS           ; yes : not do clear keyb buffer
 clr.w   (tios::kb_vars+$1C)    ; clear keyboard buffer
 bsr idle_loop
PauseShowStatRTS:
 rts


; we prepare the screen to put piece statistics
PausePreShowStat:
 move.w  timer(PC),d6
 move.l  #2560,-(a7)
 jsr tios::HeapAlloc
 lea	 4(a7),a7
 move.w  d0,d7
; handle_ptr d0,a0
 move.w d0,-(a7)
 jsr tios::HeapDeref
 addq.l #2,a7
 move.l  a0,a3

 move.l  graphlib::plane0,a1
 move.l  graphlib::plane1,a2
 moveq	 #100,d1		        ; nb of lines to erase
P_MoveRow:
 moveq	 #5,d2
P_MoveWord:
 move.w  (a1),(a0)+
 clr.w	 (a1)+
 move.w  (a2),(a0)+
 clr.w	 (a2)+
 dbra	 d2,P_MoveWord
 lea	 18(a1),a1
 lea	 18(a2),a2
 dbra	 d1,P_MoveRow

 bsr PauseShowStat              ; show piece statistics
 st	 intflag

 move.l  graphlib::plane0,a1
 move.l  graphlib::plane1,a2
 moveq	 #100,d1		        ; nb of lines to restore
P_MoveRowB:
 moveq	 #5,d2
P_MoveWordB:
 move.w  (a3)+,(a1)+
 move.w  (a3)+,(a2)+
 dbra	 d2,P_MoveWordB
 lea	 18(a1),a1
 lea	 18(a2),a2
 dbra	 d1,P_MoveRowB

 move.w  d7,-(a7)
 jsr tios::HeapFree
 addq	 #2,a7
 jsr ShowInfoText
 move.w  d6,timer
 rts

; didn't quite work -> it now turns off the calc
; and pause the game when player turn it on
TeacherKey:
 move.b  #2,ispaused            ; set pause flag for PauseShowStat
 bsr PausePreShowStat
 clr.b   ispaused               ; clear pause flag
 bra EndKeyCheck

; turn off the calc
ShutDown:
 trap #4
 rts

ShowCurPiece:
 movem.l d0-d1/d6/a0,-(a7)
 lea	 curp(PC),a0
 move.w  x(PC),d0
 move.w  y(PC),d1
 move.w  curPN(PC),d6
 bsr ShowPiece
 movem.l (a7)+,d0-d1/d6/a0
 rts

RemoveCurPiece:
 movem.l d0-d1/d6/a0,-(a7)
 lea	 curp(PC),a0
 move.w  x(PC),d0
 move.w  y(PC),d1
 move.w  curPN(PC),d6
 bsr RemovePiece
 movem.l (a7)+,d0-d1/d6/a0
 rts

UncrunchCurPiece:
 movem.l d0-d1/a0,-(a7)
 lea	 curp(PC),a0
 move.w  curPN(PC),d0
 move.w  rot(PC),d1
 bsr Uncrunch
 movem.l (a7)+,d0-d1/a0
 rts

SendBarInfo:                    ; a5 = ?
 move.l  a5,a0
 lea	 35(a0),a0
 moveq	 #16,d0
SBI_CheckRow:
 moveq	 #9,d1
SBI_CheckCol
 tst.b	 0(a0,d1)
 bne TileFound
 dbra	 d1,SBI_CheckCol
 lea	 16(a0),a0
 dbra	 d0,SBI_CheckRow
TileFound:
 add.w	 #LinkUpdateBar,d0      ; set code control
 bsr SendByte                   ; send to other calc to refresh link high
 rts

ShowBar:                        ; d1 = high of lines in linked calc
 moveq	 #15,d2
 move.l  graphlib::plane0,a2
 move.l  graphlib::plane1,a3
 lea     12(a2),a2
 lea     12(a3),a3              ; add.w #12,a3
RepShowBar:
 lea	 bargfx(PC),a0          ; load empty box GFX
 cmp.w	 d1,d2
 bge EmptyBox
 addq    #6,a0                  ; adjust to filled box GFX
EmptyBox:
 move.l  a0,a4                  ; a0 = ptr on GFX (8 * 6) to display
 move.l  a2,a1
 bsr PutSprite6
 move.l  a4,a0
 move.l  a3,a1
 bsr PutSprite6
 lea     180(a2),a2
 lea     180(a3),a3
 dbra	 d2,RepShowBar
 rts

; clear the piece in bottom-right corner
ClearNextPiece:
 lea	 nextp(PC),a0
 moveq	 #16,d0
 moveq	 #14,d1
 bsr RemovePiece
 rts

NewPiece:
 bsr ClearNextPiece
 move.w  #6,x
 clr.w	 y
 clr.w	 rot
 move.w  nextPN(PC),curPN

; update for statistics about piece given
 movem.l d0/a0,-(a7)            ; save regs
 lea	 PieceID0(PC),a0
 move.w  curPN,d0               ; d0 = piece ID
 add     d0,a0		            ; double add cuz values are 16bits, d0={0,1,2,3,4,5,6}
 add     d0,a0
 addq.w	 #1,(a0)
 movem.l (a7)+,d0/a0            ; restore regs

 bsr UncrunchCurPiece
 lea	 curp(PC),a0
 bsr ShowCurPiece

RandomNewPiece:
 moveq	 #7,d0
 bsr random
 move.w  curPN,d1
 cmp.w   d0,d1
 beq RandomNewPiece             ; avoid to have 2x the same piece


 include "tetdbg2.asm"


 move.w  d0,d6
 move.w  d0,nextPN              ; store in nextPN variable
 clr.l	 d1
 lea	 nextp(PC),a0
 bsr Uncrunch
 moveq	 #16,d0
 moveq	 #14,d1                 ; the Y position of the next piece
 bsr ShowPiece
 move.w  levtimer(PC),timer
 add.w	 #20,timer
 st	 downflag
 clr.l	 ltimer
 rts

; I changed some stuff here, but I forgot most of it...
ShowPiece:    ; shows the piece no d6, (a0) at d0,d1
 movem.l d0-d5/a0-a4,-(a7)
 move.l  graphlib::plane0,a1
 move.l  graphlib::plane1,a2
 lea	 pgfx(PC),a3
 move.w  d6,d2
 mulu.w  #12,d2
 adda.w  d2,a3
 cmp.w	 #6,d6
 bne StartPut
 cmp.w	 #3,(a0)
 bne StartPut
 lea     48(a3),a3
StartPut:
 subq	 #2,d0
 subq	 #2,d1
 moveq	 #3,d4
SP_PutSq:
 move.w  d0,d2
 move.w  d1,d3
 add.w	 (a0)+,d3
 add.w	 (a0)+,d2
 btst	 #15,d3
 bne SP_SkipSq
; changed for 6 rows again
 mulu	 #180,d3
 add.w	 d2,d3
 move.l  a3,a4
; 6 rows, not 8
 moveq  #5,d5
SP_PutRow:
 move.b  (a4)+,0(a1,d3)
 move.b  5(a4),0(a2,d3)
 add.w	 #30,d3
 dbra	 d5,SP_PutRow
SP_SkipSq:
 cmp.w	 #6,d6
 bne SP_NextSq
 lea     12(a3),a3
SP_NextSq:
 dbra	 d4,SP_PutSq
 movem.l (a7)+,d0-d5/a0-a4
 rts

; this has the same changes as the previous routine
RemovePiece:                    ; removes the piece at (a0) at d0,d1
 movem.l d0-d5/a0-a2,-(a7)
 move.l  graphlib::plane0,a1
 move.l  graphlib::plane1,a2
 subq	 #2,d0
 subq	 #2,d1
 moveq	 #3,d4
RP_ClrSq:
 move.w  d0,d2
 move.w  d1,d3
 add.w	 (a0)+,d3
 add.w	 (a0)+,d2
 btst	 #15,d3
 bne RP_SkipSq
 mulu	 #180,d3                ; changed to 180
 add.w	 d2,d3
 moveq #5,d5                    ; changed from 7 to 5
RP_PutRow:
 clr.b	 0(a1,d3)
 clr.b	 0(a2,d3)
 add.w	 #30,d3
 dbra	 d5,RP_PutRow
RP_SkipSq:
 dbra	 d4,RP_ClrSq
 movem.l (a7)+,d0-d5/a0-a2
 rts

Uncrunch:                       ; uncrunch piece d0, rot d1 into (a0)
 movem.l d0-d2/a0-a1,-(a7)
 lea	 pieces(PC),a1
 lsl.w	 #3,d0
 lsl.w	 #1,d1
 add.w	 d1,d0
 move.w  0(a1,d0),d0
 moveq	 #7,d1
GetBits:
 move.w  d0,d2
 lsr.w	 #2,d0
 and.w	 #$03,d2
 move.w  d2,(a0)+
 dbra	 d1,GetBits
 movem.l (a7)+,d0-d2/a0-a1
 rts

NewGame:
 bsr ClrScr
 moveq	 #17,d0
 move.l  a5,a1
ClrBoard:
 move.l  #$01010100,(a1)+
 clr.l	 (a1)+
 clr.l	 (a1)+
 move.l  #$00010101,(a1)+
 dbra	 d0,ClrBoard
 moveq	 #7,d0
BBorder:
 move.w  #$0101,(a1)+
 dbra	 d0,BBorder
 tst.w	 twopl                  ; 2P flag is set ?
 beq StartGame                  ; no : 1P mode => start game

; following lines are required for link sync in 2P mode
 lea	 linkdialog(PC),a6
 bsr ShowDialog                 ; display dialog
 SetFont #0
 WriteStr #40,#54,#4,canceltxt
 bsr CopyScreen
 st intflag
TrySync:
 move.b   #LinkNewConn,d0
 bsr SendByte
TrySyncTimer:
 move.w   #LinkDelay,timer      ; LinkDelay int1 ticks (500)
LoopReceive:
 bsr ReceiveByte
 move.l   d0,d1                 ; copy byte received
 and.b    #$FF,d0
 tst.b    d0
 beq LinkKeybTest               ; if no byte received => test for ESC
 bsr SendByte                   ; echo request !! allaow future versions
 cmp.b    #LinkNewConn,d0
 beq SyncDone                   ; link etablished
 cmp.b    #LinkOldConn,d0
 beq SyncDoneOldVersion         ; link etablished but old version
LinkKeybTest:
 bsr GetKeyStat                 ; test if ESC is pressed
 bsr KeyTrans
 cmp.b    #KeyESC,KeyEqu
 beq LinkError                  ; user aborts => jump to error handler
 cmp.w    #0,timer
 bgt LoopReceive                ; loop on receive datas
 bra TrySync                    ; loop on sending new top sync

LinkError:                      ; aborted by user
 move.b   #LinkReset,d0
 bsr SendByte                   ; abort on the linked calc
 bsr ReceiveByte                ; clear buffer
 addq #4,a7                     ; required but why ??
 sf intflag
 clr.w   (tios::kb_vars+$1C)
 bsr idle_loop
 bra MainRepeatMenu             ; jump to main menu

SyncDoneOldVersion:
 bsr SendByte                   ; send again confirmation code for old versions
 lea     oldverdialog(PC),a6    ; calc is a 92+ then warn the user
 bsr ShowDialog
 SetFont #0
 WriteStr #23,#48,#4,badlinkver
 WriteStr #40,#56,#4,canceltxt
 bsr CopyDialogBox

TryOldSync:                     ; re-sync for old version of Tetris (link was too bad)
 move.b   #LinkOldConn,d0
 bsr SendByte
 move.w   #LinkDelay,timer      ; LinkDelay int1 ticks (500)
LoopOldReceive:
 bsr ReceiveByte
 move.l   d0,d1                 ; copy byte received
 and.b    #$FF,d0
 cmp.b    #LinkOldConn,d0
 beq SyncDone                   ; link etablished but old version
 bsr GetKeyStat                 ; test if ESC is pressed
 bsr KeyTrans
 cmp.b    #KeyESC,KeyEqu
 beq LinkError                  ; user aborts => jump to error handler
 cmp.w    #0,timer
 bgt LoopOldReceive             ; loop on receive datas
 bra TryOldSync                 ; loop on sending new top sync

SyncDone:
Start2PGame:
 bsr ClrScr
 clr.l	 d1
 bsr ShowBar


; here is the init for start a new game
StartGame:
; starts by init statistics variables
 clr.l	 score
 clr.w	 Sline
 clr.w	 Dline
 clr.w	 Tline
 clr.w	 TTline
 clr.w   maxlines
 clr.w	 lines
 clr.w   PieceID0
 clr.w   PieceID1
 clr.w   PieceID2
 clr.w   PieceID3
 clr.w   PieceID4
 clr.w   PieceID5
 clr.w   PieceID6

 st	 intflag
 bsr ShowBrickWall
 move.w  high(PC),d1
 beq NoTrashRows
 lsl.w	 #1,d1
 subq	 #1,d1
TrashRows:
 bsr TrashRow
 dbra	 d1,TrashRows
NoTrashRows:
 bsr RandomNewPiece
 bsr NewPiece

; Warning : if 2P we can't be in B mode
 tst.w   twopl                  ; twopl = 0 ?
 beq endadjustbmode             ; Yes : 1P => jump
 clr.w   bmode				    ; 2P : B mode not possible

endadjustbmode:
 tst.w	 bmode                  ; bmode = 0 ?
 beq endadjustNBline            ; Yes : not in B mode => init lines

 movem.l d0-d2,-(a7)	    ; save regs
; B mode : calculate number of line to do ...
; maxlines=(10-level)*10+(5-high)*10+15 
 clr.l   d0	                    ; (10-level)*10 -> d0
 move.w  #10,d0				
 sub.w   level,d0
 mulu	 #10,d0
 clr.l	 d1	                    ; (5-high)*10 -> d1
 move.w  #5,d1
 sub.w   high,d1
 mulu    #10,d1
 add.w	 d1,d0				    ; d0=d0+d1
 add.w   #15,d0                 ;           +15
 move.w  d0,maxlines
 movem.l (a7)+,d0-d2            ; restore regs
endadjustNBline:

 lea	 levels(PC),a0
 move.w  level(PC),d0
 move.w  d0,st_lev
 lsl.w	 #1,d0
 move.w  0(a0,d0),levtimer

ShowInfoText:
; font type changed, all the x and y positions were changed again
 SetFont #0
 WriteStr #110,#2,#4,Slinetxt
 WriteStr #110,#8,#4,Dlinetxt
 WriteStr #110,#14,#4,Tlinetxt
 WriteStr #110,#20,#4,TTlinetxt
 SetFont #1
 WriteStr #105,#30,#4,leveltxt
 WriteStr #105,#40,#4,linestxt
 WriteStr #105,#60,#4,scoretxt
 bsr ShowScore
 rts

WriteChar:
 movem.l d0-d2/a0-a1,-(a7)
 move.w  #$00FF,-(a7)
 clr.w	 -(a7)
 move.w  #$00FF,-(a7)
 move.w  #4,-(a7)
 move.w  d1,-(a7)
 move.w  d0,-(a7)
 move.w  d2,-(a7)
;; jsr tios::DrawCharXY
 bsr DrawCharXY
 lea	 14(a7),a7
 movem.l (a7)+,d0-d2/a0-a1
 rts

; CONVERTED!!! 5/15/98!!
Menu:
 bsr ShowTetrisLogo
 SetFont #2
 move.l graphlib::plane0,a0
 lea    46*30+1(a0),a0
 moveq	 #32,d0
VLines:
 moveq	 #9,d1
 move.l  a0,a1
VLine:
 move.b  #$80,(a1)
 cmp.b	 #4,d1
 addq	 #2,a1
 dbra	 d1,VLine
 lea	 30(a0),a0
 dbra	 d0,VLines
 move.l graphlib::plane0,a0
 lea 46*30+1(a0),a0
 moveq	 #2,d0
HLines:
 moveq	 #17,d1
HLine:
 move.w  d1,d2
 sub.w	 #6,d2
 asr.w	 #1,d2
 beq SkipHLine
 st	 (a0)
SkipHLine:
 addq	 #1,a0
 dbra	 d1,HLine
 lea	 462(a0),a0
 dbra	 d0,HLines
 moveq	 #12,d0
 moveq	 #50,d1
 moveq	 #48,d2
 moveq	 #9,d3
PutLDig:
 bsr WriteChar
 add.w	 #16,d0
 addq	 #1,d2
 cmp.w	 #53,d2
 bne LSameRow
 moveq	 #12,d0
 moveq	 #66,d1
LSameRow:
 dbra	 d3,PutLDig
 move.w  #108,d0
 moveq	 #50,d1
 moveq	 #48,d2
 moveq	 #5,d3
PutRDig:
 bsr WriteChar
 add.w	 #16,d0
 addq	 #1,d2
 cmp.w	 #51,d2
 bne RSameRow
 move.w  #108,d0
 moveq	 #66,d1
RSameRow:
 dbra	 d3,PutRDig
 WriteStr #30,#35,#4,leveltxt
 WriteStr #115,#35,#4,hightxt
 SetFont #1
 WriteStr #12,#80,#4,tptxt

 move.l graphlib::plane0,a0
 lea    88*30(a0),a1
 move.l #$000FE000,(a1)	        ; underline T letter
 bsr CopyScreen                 ; keep a copy of the screen for gray scales

; this text will appear in gray but it can be erased 
 lea    99*30+4(a0),a1
 move.w #$0FE0,(a1)		        ; underline S letter

 tst.w	 twopl		            ; 2Player ?
 beq trybmode                   ; no
try1rowless:
 WriteStr #12,#91,#4,ltxt
 bra trychoiceend
trybmode:
 WriteStr #12,#91,#4,btxt
trychoiceend:

 LCD_Copy_Zone8 graphlib::plane0,graphlib::plane1,#1,#91,#19,#99
 bsr ShowFlags
 move.w  st_lev(PC),d0
 move.w  d0,d1
 moveq	 #9,d6
 moveq	 #5,d5
 bsr Mark
 move.w  high(PC),d0
 moveq	 #5,d6
 moveq	 #3,d5
 bsr Mark
 clr.l	 d7
ChooseLevel:
 moveq	 #5,d5
 moveq	 #9,d6
ResTimer:
 move.w  #150,timer
WaitKey:
 tst.w	 timer
 bne NoFlash
 move.w  #150,timer
 move.w  d1,d0
 bsr Mark
NoFlash:
; i changed the key addresses to Plusshell convention here
 tst.w   (tios::kb_vars+$1C)
 beq WaitKey
 move.w  d1,d2

 move.w  (tios::kb_vars+$1E),d0
 clr.w   (tios::kb_vars+$1C)
 bsr Idle_loop_convert          ; conversion for 92+

; some of the following keycodes may have been changed, these are the
; standard codes listed in the manual unlike the others in the program
 cmp.w	 #337,d0
 bne CDown
 cmp.w	 d5,d1
 bcs WaitKey
 sub.w	 d5,d1
 bra Update
CDown:
 cmp.w	 #340,d0
 bne CLeft
 cmp.w	 d5,d1
 bcc WaitKey
 add.w	 d5,d1
 bra Update
CLeft:
 cmp.w	 #338,d0
 bne CRight
 tst.w	 d1
 beq WaitKey
 subq	 #1,d1
 bra Update
CRight:
 cmp.w	 #344,d0
 bne COther
 cmp.w	 d6,d1
 beq WaitKey
 addq	 #1,d1
Update:
 tst.w	 d7
 bne NoRemove
 move.w  d2,d0
 bsr Mark
NoRemove:
 move.w  d1,d0
 bsr Mark
 clr.l	 d7
 move.w  #150,timer
 bra WaitKey
COther:
 cmp.w	 #264,d0                ; (ESC)
 beq Return
 cmp.w	 #51,d0                 ; (3) Send one row less / Select B mode
 beq ChangeLLFlag
 cmp.w	 #115,d0                ; (s) Send one row less / Select B mode
 beq ChangeLLFlag
 cmp.w	 #116,d0                ; (t) Two players
 beq ChangeTPFlag
 cmp.w	 #257,d0                ; (backspace)
 bne CEnter
 cmp.w	 #9,d6
 beq WaitKey
 tst.w	 d7
 beq ChooseLev
 move.w  d1,d0
 bsr Mark
ChooseLev:
 move.w  d1,high
 move.w  level(PC),d1
 bra ChooseLevel
CEnter:
 cmp.w	 #13,d0
 bne WaitKey
 cmp.w	 #9,d6
 bne HighChosen
 move.w  d1,level
 tst.w	 d7
 beq ChooseHigh
 move.w  d1,d0
 bsr Mark
ChooseHigh:
 moveq	 #3,d5
 moveq	 #5,d6
 move.w  high(PC),d1
 bra ResTimer
HighChosen:
 move.w  d1,high
Return:
 rts

ChangeLLFlag:
 tst.w	 twopl                  ; 2PL = 0 ?
 beq ChangeLLFlag01             ; Yes : 1PL
 eori.w  #8,llflag              ; llflag xor = 8
 bra ChangeLLFlag02
ChangeLLFlag01:
 eori.w  #8,bmode               ; bmode xor = 8
ChangeLLFlag02:
 bsr ShowFlags
 bra WaitKey

; update of the field for Send 1 row less / Select B mode ...
ChangeTPFlag:
 movem.l d0-d1/a0-a1,-(a7)
 tst.w	 twopl                  ; 2P = 0 ?
 bne ChangeTPFlag_trybmode      ; yes : 1P
ChangeTPFlag_try1rowless:
 WriteStr #12,#91,#4,ltxt
 bra ChangeTPFlag_trychoiceend
ChangeTPFlag_trybmode:
 WriteStr #12,#91,#4,btxt
ChangeTPFlag_trychoiceend:
 LCD_Copy_Zone8 graphlib::plane0,graphlib::plane1,#1,#91,#19,#99
 movem.l (a7)+,d0-d1/a0-a1
 eori.w  #8,twopl               ; twopl xor = 8
 bsr ShowFlags
 bra WaitKey


PutSprite:
 moveq	 #7,d0
PS_PutRow:
 move.b  (a0)+,(a1)
 lea	 30(a1),a1
 dbra	 d0,PS_PutRow
 rts

PutSprite6:
 moveq	 #5,d0
PS_PutRow6:
 move.b  (a0)+,(a1)
 lea	 30(a1),a1
 dbra	 d0,PS_PutRow6
 add.w #2,a0
 rts

ShowFlags:
 lea	 box(PC),a0
 add.w	 twopl(PC),a0
 move.l  a0,a2
 move.l  graphlib::plane1,a3
 move.l  graphlib::plane0,a1
 lea 2400(a1),a1
 bsr PutSprite
 add.w	 #2400,a3
 move.l  a3,a1
 move.l  a2,a0
 bsr PutSprite
 lea	 box(PC),a0
; test of the choice in 2nd line ...
 tst.w   twopl                  ; 2P = 0 ?
 bne _try1rowless               ; no => 2P
_trybmode:
 add.w   bmode(PC),a0           ; 1P mode
 bra _trychoiceend
_try1rowless:
 add.w   llflag(PC),a0          ; 2P mode
_trychoiceend:
 move.l  a0,a2
 move.l  graphlib::plane1,a3
 move.l  graphlib::plane0,a1
 lea 2730(a1),a1
 bsr PutSprite
 add.w	 #2730,a3
 move.l  a3,a1
 move.l  a2,a0
 bsr PutSprite
 rts

Mark:
 movem.l d0-d1/a0,-(a7)
 bchg	 #0,d7
 move.l graphlib::plane0,a0
 lea 47*30+1(a0),a0
 cmp.w	 #9,d6
 beq MarkLevel
 lea	 12(a0),a0
MarkLevel:
 cmp.w	 d5,d0
 bcs MarkSquare
 sub.w	 d5,d0
 lea	 480(a0),a0
MarkSquare:
 lsl.w	 #1,d0
 moveq	 #14,d1
ML_MarkRow:
 eori.b  #$7F,0(a0,d0)
 eori.b  #$FF,1(a0,d0)
 lea	 30(a0),a0
 dbra	 d1,ML_MarkRow
 movem.l (a7)+,d0-d1/a0
 rts

; copy plane0 to plane1 to simulate gray2 in gray4
CopyScreen:
 movem.l d0/a0-a1,-(a7)
 move.l  graphlib::plane0,a0
 move.l  graphlib::plane1,a1
 move.w  #959,d0
RepCopy:
 move.l  (a0)+,(a1)+
 dbra	 d0,RepCopy
 movem.l (a7)+,d0/a0-a1
 rts

Titel:
 bsr ShowTetrisLogo
 SetFont #2
 WriteStr #28,#55,#4,top3txt

; i changed the credits .. again
 SetFont #1
 WriteStr #8,#44,#4,author2
 SetFont #0
 WriteStr #8,#37,#4,author1

 WriteStr #0,#68,#4,tranktxt
 WriteStr #40,#68,#4,tnametxt
 WriteStr #108,#68,#4,tscoretxt
 WriteStr #133,#68,#4,tlinetxt
 WriteStr #148,#68,#4,tleveltxt

 lea	 hiscore(PC),a2
 clr.l   d0
 moveq	 #76,d1
 move.w  #$0031,d2
 moveq	 #2,d3
ShowHiscore:
; display position 
 movem.l d0-d7,-(a7)
 SetFont #1
 movem.l (a7)+,d0-d7
 bsr WriteChar
 move.l  d1,d4
 movem.l d0-d2,-(a7)
; display name
 WriteStrA #10,d1,#4,a2         ; a2 = ptr on name
; display score
 movem.l d0-d7,-(a7)
 SetFont #0
 movem.l (a7)+,d0-d7
 lea	 tmpstr+6(PC),a0
 moveq	 #6,d1
 move.l  16(a2),d0
 bsr ConvStr
 WriteStrA #105,d4,#4,a0
; display lines
 lea	 tmpstr+4(PC),a0
 moveq	 #4,d1
 move.l  20(a2),d0
 bsr ConvStr
 WriteStrA #130,d4,#4,a0
; display level
 lea	 tmpstr+4(PC),a0
 moveq	 #4,d1
 move.l  24(a2),d0
 bsr ConvStr
 WriteStrA #145,d4,#4,a0
 movem.l (a7)+,d0-d2
 lea	 28(a2),a2              ; adjust for next name structure
 addq	 #1,d2
 addq	 #8,d1
 dbra	 d3,ShowHiscore
 bsr CopyScreen
 rts

ShowTetrisLogo:
 bsr ClrScr
 move.l  graphlib::plane0,a0
 lea	 logo(PC),a1
 moveq   #33,d0                 ; # of lines-1
ShowLogo:
 moveq   #9,d1                  ; # of word to copy-1
ShowLogoRow:
 move.w  (a1)+,(a0)+            ; copy into screen location
 dbra	 d1,ShowLogoRow
 lea	 10(a0),a0              ; skip 10 16bits words (=160 bits = 240-180)
 dbra	 d0,ShowLogo
 rts

ShowDialog:
 jsr graphlib::show_dialog
CopyDialogBox:
 move.l  graphlib::plane0,a0
 lea     $1C*30+2(a0),a0
 move.l  graphlib::plane1,a1
 add.w	 #$1C*30+2,a1
 moveq	 #39,d0
SD_CopyRow:
 moveq	 #13,d1
SD_CopyByte:
 move.b  (a0)+,(a1)+
 dbra	 d1,SD_CopyByte
 lea	 16(a0),a0
 lea	 16(a1),a1
 dbra	 d0,SD_CopyRow
 rts

ClrScr:
 movem.l d0/a0-a1,-(a7)
 move.l  graphlib::plane0,a0
 move.l  graphlib::plane1,a1
 move.w  #959,d0
RepClr:
 clr.l	 (a0)+
 clr.l	 (a1)+
 dbra	 d0,RepClr
 movem.l (a7)+,d0/a0-a1
 rts

GetKeyStat:
 movem.l d0-d1/a0,-(a7)
 lea	 keystat(PC),a0
 move.w  #$FFFE,d0
 moveq	 #9,d1                  ; changed to 9 for 92p compatibility
GetKeys:
 GetKey  d0,(a0)+
 rol.w	 #1,d0
 dbra	 d1,GetKeys
 movem.l (a7)+,d0-d1/a0
 rts

ShowBrickWall:
 move.l  graphlib::plane0,a0
 move.l  graphlib::plane1,a1
; this got changed so the walls are on the side of the screen not the middle
 moveq	 #11,d0
PutBrickWall:
 lea	 brickgfx(PC),a2
 moveq	 #7,d1
PBWRow:
 move.b  (a2),(a0)
 move.b  8(a2),(a1)
 move.b  (a2)+,11(a0)
 move.b  7(a2),11(a1)
 lea	 30(a0),a0
 lea	 30(a1),a1
 dbra	 d1,PBWRow
 dbra	 d0,PutBrickWall
 rts

SendByte:
 movem.l d0-d1/a0-a1,-(a7)
 move.b  d0,linkbuf
 move.w  #1,-(a7)
 pea	 linkbuf(PC)
 jsr tios::transmit
 addq	 #6,a7
 movem.l (a7)+,d0-d1/a0-a1
 rts

ReceiveByte:
 movem.l d1/a0-a1,-(a7)
 clr.b	 linkbuf
 move.w  #1,-(a7)
 pea	 linkbuf(PC)
 jsr tios::receive
 addq	 #6,a7
 clr.l	 d0
 move.b  linkbuf(PC),d0
 movem.l (a7)+,d1/a0-a1
 rts

; this is the new int 1 handler
Int1Handler:
 tst.w	 timer
 beq DecLTimer
 subq.w  #1,timer
DecLTimer:
 tst.w	 ltimer
 beq DecRTimer
 subq.w  #1,ltimer
DecRTimer:
 tst.w	 rtimer
 beq CheckFlag
 subq.w  #1,rtimer
CheckFlag:
 tst.b   intflag
 bne EndInt
 move.l  oldint1(PC),-(a7)
 rts
EndInt:
 rte


; adjust util::idle_loop of 92p to 89 (up, down, left, right)
Idle_loop_convert:
 tst.b   is92p                  ; is92p = 0 ?
 bne Idle_loop_convert92p       ; no : this is a 92p
 rts
Idle_loop_convert92p:
 cmp.w	 #337,d0
 beq Idle_loop_convert_Up
 cmp.w	 #340,d0
 beq Idle_loop_convert_Down
 cmp.w	 #338,d0
 beq Idle_loop_convert_Left
 cmp.w	 #344,d0
 beq Idle_loop_convert_Right
 rts


Idle_loop_convert_Up:
 move.w  #338,d0
 rts
Idle_loop_convert_Down:
 move.w  #344,d0
 rts
Idle_loop_convert_Left:
 move.w  #337,d0
 rts
Idle_loop_convert_Right:
 move.w  #340,d0
 rts



; translate from key map to key ID (see tetmacr.h)
KeyTrans:
 clr.b   KeyEqu
 tst.b   is92p                  ; is92p = 0 ?
 bne Trans92p                   ; no : this is a 92p

; translation for 89
 btst.b  #0,keystat             ; (Up)
 beq LblKeyUp
 btst.b  #4,keystat             ; (2nd)
 beq LblKey2nd
 btst.b  #5,keystat             ; (Shift)
 beq LblKeyShift
 btst.b  #2,keystat             ; (Down)
 beq LblKeyDown
 btst.b  #1,keystat             ; (Left)
 beq LblKeyLeft
 btst.b  #3,keystat             ; (Right)
 beq LblKeyRight
 btst.b  #6,keystat             ; (Diamond)
 beq LblKeyDiamond
 btst.b  #7,keystat             ; (Alpha)
 beq LblKeyAlpha
 btst.b  #0,keystat+6           ; (ESC)
 beq LblKeyESC
 btst.b  #6,keystat+5           ; (Home)
 beq LblKeyHome
 btst.b  #7,keystat+5           ; (F1)
 beq LblKeyF1
 btst.b  #7,keystat+4           ; (F2)
 beq LblKeyF2
 btst.b  #7,keystat+3           ; (F3)
 beq LblKeyF3
 btst.b  #7,keystat+2           ; (F4)
 beq LblKeyF4
 btst.b  #0,keystat+1           ; (ENTER)
 beq LblKeyENTER
 rts

; translation for 92p
Trans92p:
 btst.b  #4,keystat             ; (Left)
 beq LblKeyLeft
 btst.b  #6,keystat             ; (Right)
 beq LblKeyRight
 btst.b  #7,keystat             ; (Down)
 beq LblKeyDown
 btst.b  #5,keystat             ; (Up)
 beq LblKeyUp
 btst.b  #4,keystat+7           ; (F1)
 beq LblKeyShift
 btst.b  #4,keystat+6           ; (F5)
 beq LblKey2nd
 btst.b  #3,keystat             ; (Hand)
 beq LblKeyDiamond
 btst.b  #2,keystat+2           ; (D)
 beq LblKeyDiamond
 btst.b  #6,keystat+6           ; (ENTER [bellow kepad])
 beq LblKeyENTER
 btst.b  #3,keystat+4           ; (T)
 beq LblKeyHome
 btst.b  #7,keystat+6           ; (P)
 beq LblKeyAlpha
 btst.b  #6,keystat+8           ; (Q)
 beq LblKeyESC
 btst.b  #3,keystat+9           ; (ESC)
 beq LblKeyESC 
 btst.b  #1,keystat+9           ; (ENTER [other])
 beq LblKeyENTER
 btst.b  #4,keystat+4           ; (F2)
 beq LblKeyF1
 btst.b  #4,keystat+2           ; (F3)
 beq LblKeyF2
 btst.b  #4,keystat+9           ; (F4)
 beq LblKeyF3
 btst.b  #4,keystat+5           ; (F6)
 beq LblKeyF4
 rts


LblKeyUp:
 move.b  #KeyUp,KeyEqu
 rts
LblKeyDown:
 move.b  #KeyDown,KeyEqu
 rts
LblKeyLeft:
 move.b  #KeyLeft,KeyEqu
 rts
LblKeyRight:
 move.b  #KeyRight,KeyEqu
 rts
LblKeyESC:
 move.b  #KeyESC,KeyEqu
 rts
LblKey2nd:
 move.b  #Key2nd,KeyEqu
 rts
LblKeyShift:
 move.b  #KeyShift,KeyEqu
 rts
LblKeyDiamond:
 move.b  #KeyDiamond,KeyEqu
 rts
LblKeyAlpha:
 move.b  #KeyAlpha,KeyEqu
 rts
LblKeyHome:
 move.b  #KeyHome,KeyEqu
 rts
LblKeyF1:
 move.b  #KeyF1,KeyEqu
 rts
LblKeyF2:
 move.b  #KeyF2,KeyEqu
 rts
LblKeyF3:
 move.b  #KeyF3,KeyEqu
 rts
LblKeyF4:
 move.b  #KeyF4,KeyEqu
 rts
LblKeyENTER:
 move.b  #KeyENTER,KeyEqu
 rts

; switch into gray2 mode, uninstall int1 hook and terminate this program
Done:
 jsr graphlib::gray2
 move.l  oldint1(PC),($040064)
 rts                            ; quit this fabulous program

; generate random number
random:
    move.l  d1,-(a7)
	move.w	rand_seed(pc),d1
	mulu.w	#31421,d1
	add.w	#6927,d1
	mulu.w	d1,d0
	move.w	d1,rand_seed
	clr.w	d0
	swap	d0
    move.l  (a7)+,d1
	rts

idle_loop:
	movem.l	d1-d7/a0-a6,-(a7)
\APD_reset
    push.w	#2
	jsr	tios::OSTimerRestart	; resets APD timer
	addq.l	#2,a7
\keyloop
	move.w	#2,-(a7)
	jsr	tios::OSTimerExpired	; if APD expired..
	addq.l	#2,a7
	tst.w	d0
	beq	\noshutdown
\shutdown
	trap	#4			; then shut down
	bra	\APD_reset
\noshutdown
	tst.w	KEY_PRESSED_FLAG	; has a key been pressed ?
	beq	\keyloop		; no..
	jsr	tios::ngetchx	; yes : get the key code
	cmp.w	#KEY_DIAMOND+$10B,d0	; Diamand+ON shuts down the calc
	beq	\shutdown
	move.w	d0,d6
	push.w	#3
	jsr	tios::ST_busy	; clears BUSY flag
	addq.l	#2,a7
	move.w	d6,d0
	movem.l	(a7)+,d1-d7/a0-a6
	rts

DrawCharXY:
    move.w 4(a7),d0
    move.b d0,charstr
    move.w 6(a7),d0
    move.w 8(a7),d1
    move.w 10(a7),d2
    move.w d2,-(a7)
    move.l #charstr,-(a7)
    move.w d1,-(a7)
    move.w d0,-(a7)
    jsr tios::DrawStrXY
    add.l #10,a7
    rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;; Data memory zone ;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


levels      dc.w 310,258,215,179,149,124,104,86,72,60      ; 3*10 level speed
            dc.w 51,40,36,32,28,24,20,16,13,10,
            dc.w 8,6,5,4,3,2,1,0,0,0
       
scoring     dc.w 40,100,300,1200

pieces      dc.w %0010011010101011,%0101011000110111
            dc.w %0001001001101010,%0101100101100111
            dc.w %0010011010100011,%0001010101100111
            dc.w %0010011010101001,%0101011001111011
            dc.w %0110101001111011,%0110101001111011
            dc.w %0110101001111011,%0110101001111011
            dc.w %0010011010100111,%0010010101100111
            dc.w %0010010101101010,%0101011001111010
            dc.w %0011011001111010,%0001001001100111
            dc.w %0011011001111010,%0001001001100111
            dc.w %0010011001111011,%0010001101010110
            dc.w %0010011001111011,%0010001101010110
            dc.w %0010011010101110,%0100010101100111
            dc.w %0010011010101110,%0100010101100111

empty       dc.w 0,0,0,0,0,0,0,0

pgfx        dc.b %11111111,%10000001,%10111101,%10100101,%10111101,%11111111
            dc.b %11111111,%11111111,%11111111,%11100111,%11111111,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            dc.b %11111111,%10000001,%10000001,%10000001,%10000001,%11111111
            
            dc.b %11111111,%10000001,%10011001,%10011001,%10000001,%11111111
            dc.b %11111111,%10000001,%10011001,%10011001,%10000001,%11111111
            dc.b %11111111,%10000001,%10000001,%10000101,%10111101,%11111111
            dc.b %11111111,%11000011,%11011111,%11011111,%11011111,%11111111
            
            dc.b %11111111,%11111111,%11111111,%11100111,%11100111,%11111111
            dc.b %11111111,%10000001,%10111101,%10100101,%10111101,%11111111
            dc.b %11111111,%10000001,%10011001,%10011001,%10000001,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            
            dc.b %11111111,%10001001,%00100001,%00000101,%01010001,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            dc.b %11111111,%10101000,%00000010,%01000000,%10000100,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            
            dc.b %11111111,%10100000,%00000010,%00100000,%10000100,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            dc.b %11111111,%10100100,%10000000,%11001000,%10000000,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            
            dc.b %10000011,%10010001,%11000101,%10010101,%11000011,%11111111
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            dc.b %10010011,%11000001,%10001011,%10100001,%10000011,%11010001
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
            
            dc.b %10010011,%11000001,%10001011,%10100001,%10000001,%10110001
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111 
            dc.b %11111111,%10001001,%10100011,%10000001,%11001001,%10000011
            dc.b %11111111,%11111111,%11111111,%11111111,%11111111,%11111111
 
box         dc.b %01111111,%01000001,%01000001,%01000001,%01000001,%01000001,%01111111,0
            dc.b %01111111,%01100011,%01010101,%01001001,%01010101,%01100011,%01111111,0

bargfx      dc.b %11000110,%10000010,%10000010,%10000010,%10000010,%11000110
            dc.b %11000110,%10000010,%10111010,%10111010,%10000010,%11000110


brickgfx    dc.b %01000100,%01000100,%01000100,%11111111,%00010001,%00010001,%00010001,%11111111
            dc.b %01011101,%11111111,%11111111,%11111111,%01110111,%11111111,%11111111,%11111111

logo
    incbin  "mainlogo.bin"

linkdialog      dc.l $0010001C,$007F0044,$0010000C,waittxt,0
oldverdialog    dc.l $0010001C,$007F0044,$00100008,oldvertxt,0
losedialog      dc.l $0010001C,$007F0044,$0013000C,gotxt,0
windialog       dc.l $0010001C,$007F0044,$001B000C,wintxt,0
abortdialog     dc.l $0010001C,$007F0044,$0008000C,aborttxt,0
ti92pdialog     dc.l $0010001C,$007F0044,$000B000C,warntxt,0

; hiscore struct name-name-name-name score-s lines-l level-l
hiscore     dc.b "---------------",0,0,0,0,0,0,0,0,0,0,0,0,0
            dc.b "---------------",0,0,0,0,0,0,0,0,0,0,0,0,0
            dc.b "---------------",0,0,0,0,0,0,0,0,0,0,0,0,0

oldint1     dc.l 0              ; old int1 vector
oldstackptr dc.l 0              ; old stack pointer

curp        ds.w 8
nextp       ds.w 8
nextptmp    ds.w 8
tmpp        ds.w 8
rows        ds.w 4

rand_seed   dc.w 0
charstr		dc.b 0

curPN       dc.w 0
nextPN      dc.w 0
x           dc.w 0
y           dc.w 0
rot         dc.w 0

score       dc.l 0
lines       dc.w 0
maxlines    dc.w 0
level       dc.w 0
high        dc.w 0

Sline       dc.w 0              ; statistics values about lines
Dline       dc.w 0
Tline       dc.w 0
TTline      dc.w 0

PieceID0    dc.w 0              ; statistics values about piece given
PieceID1    dc.w 0
PieceID2    dc.w 0
PieceID3    dc.w 0
PieceID4    dc.w 0
PieceID5    dc.w 0
PieceID6    dc.w 0

st_lev      dc.w 0
st_high     dc.w 0
disUflag    dc.w 0
disDflag    dc.w 0
llflag      dc.w 0

twopl       dc.w 0
bmode       dc.w 0


 include "tetdbg3.asm"


is92p       dc.b 0              ; flags are byte sized
ispaused    dc.b 0
isoldver    dc.b 0
KeyEqu      dc.b 0              ; key translation in equ (see tetmacr.h)

timer       dc.w 0
ltimer      dc.w 0
rtimer      dc.w 0
levtimer    dc.w 0

tmpstr      ds.b 10
keystat     ds.b 10

intflag     dc.b 0
downflag    dc.b 0
upflag      dc.b 0
linkbuf     dc.b 0

timer_value     dc.w timerHW1          ; timer values for same speed on HW1 and HW2
ltimer_value    dc.w ltimerHW1
rtimer_value    dc.w rtimerHW1

; these text are bold in dialog box
waittxt     dc.b "Waiting...",0
gotxt       dc.b "Game over",0
wintxt      dc.b "You win",0
warntxt     dc.b "Not a TI-89",0
oldvertxt   dc.b "Warning !!",0
aborttxt    dc.b "Game aborted",0

leveltxt    dc.b "Level",0
linestxt    dc.b "Lines",0
scoretxt    dc.b "Score",0
hightxt     dc.b "High",0

statistxt   dc.b "Statistics",0
pausetxt    dc.b "Game paused",0
entertxt    dc.b "Press ENTER to continue",0
canceltxt   dc.b "Press ESC to abort",0
badcalctxt  dc.b "Picture will not fit to screen",0
badlinkver  dc.b "Remote Tetris is old or cheated",0
totaltxt    dc.b "Total pieces",0

tranktxt    dc.b "Rank",0
tnametxt    dc.b "Name",0
tscoretxt   dc.b "Score",0
tlinetxt    dc.b "Lns",0
tleveltxt   dc.b "Lvl",0


Slinetxt    dc.b "Single :",0
Dlinetxt    dc.b "Double :",0
Tlinetxt    dc.b "Triple :",0
TTlinetxt   dc.b "Tetris :",0

tptxt       dc.b "Two players mode (link)",0
ltxt        dc.b "2P: Send one row less",0
btxt        dc.b "1P: Select B mode    ",0

; split author names into two lines
author1     dc.b "by Jimmy Mrdell, ported by Joel Jordan",0
author2     dc.b "Upgraded by Cyril Cleaud",0

top3txt     dc.b "* TOP THREE *",0
ent1txt     dc.b "CONGRATULATIONS!",0
ent2txt     dc.b "You are in the TOP THREE !",0
ent3txt     dc.b "Enter your name",0
ent4txt     dc.b "Your score :       ",0

_comment    dc.b "Tetris 89/92+ v1.40 - Jan 20, 2005",0

	BSS
board ds.b  $200

 end